@@ -17,13 +17,14 @@
  *
  */
 
+#include "config.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <errno.h>
@@ -41,10 +42,15 @@ struct BaconMessageConnection {
 	/* The socket path itself */
 	char *path;
 
-	/* FD is for the connection, serverfd is for the server socket,
-	 * it accepts incoming connections. */
-	int fd, serverfd, server_conn_id, conn_id;
+	/* File descriptor of the socket */
+	int fd;
+	/* Channel to watch */
 	GIOChannel *chan;
+	/* Event id returned by g_io_add_watch() */
+	int conn_id;
+
+	/* Connections accepted by this connection */
+	GSList *accepted_connections;
 
 	/* callback */
 	void (*func) (const char *message, gpointer user_data);
@@ -65,12 +71,31 @@ test_is_socket (const char *path)
 	return FALSE;
 }
 
+static gboolean
+is_owned_by_user_and_socket (const char *path)
+{
+	struct stat s;
+
+	if (stat (path, &s) == -1)
+		return FALSE;
+
+	if (s.st_uid != geteuid ())
+		return FALSE;
+
+	if ((s.st_mode & S_IFSOCK) != S_IFSOCK)
+		return FALSE;
+	
+	return TRUE;
+}
+
 static gboolean server_cb (GIOChannel *source,
 			   GIOCondition condition, gpointer data);
 
 static gboolean
 setup_connection (BaconMessageConnection *conn)
 {
+	g_return_val_if_fail (conn->chan == NULL, FALSE);
+
 	conn->chan = g_io_channel_unix_new (conn->fd);
 	if (!conn->chan) {
 		return FALSE;
@@ -81,19 +106,38 @@ setup_connection (BaconMessageConnection *conn)
 	return TRUE;
 }
 
+static void
+accept_new_connection (BaconMessageConnection *server_conn)
+{
+	BaconMessageConnection *conn;
+	int alen;
+
+	g_return_if_fail (server_conn->is_server);
+
+	conn = g_new0 (BaconMessageConnection, 1);
+	conn->is_server = FALSE;
+	conn->func = server_conn->func;
+	conn->data = server_conn->data;
+
+	conn->fd = accept (server_conn->fd, NULL, (guint *)&alen);
+
+	server_conn->accepted_connections =
+		g_slist_prepend (server_conn->accepted_connections, conn);
+
+	setup_connection (conn);
+}
+
 static gboolean
 server_cb (GIOChannel *source, GIOCondition condition, gpointer data)
 {
 	BaconMessageConnection *conn = (BaconMessageConnection *)data;
 	char *message, *subs, buf;
-	int cd, alen, rc, offset;
+	int cd, rc, offset;
 	gboolean finished;
 
 	offset = 0;
-	if (conn->serverfd == g_io_channel_unix_get_fd (source)) {
-		cd = accept (conn->serverfd, NULL, (guint *)&alen);
-		conn->fd = cd;
-		setup_connection (conn);
+	if (conn->is_server && conn->fd == g_io_channel_unix_get_fd (source)) {
+		accept_new_connection (conn);
 		return TRUE;
 	}
 	message = g_malloc (1);
@@ -138,22 +182,67 @@ server_cb (GIOChannel *source, GIOCondition condition, gpointer data)
 }
 
 static char *
-socket_filename (const char *prefix)
+find_file_with_pattern (const char *dir, const char *pattern)
 {
-	char *filename, *path;
-	const char *dir;
+	GDir *filedir;
+	char *found_filename;
+	const char *filename;
+	GPatternSpec *pat;
 
-	filename = g_strdup_printf (".%s.%s", prefix, g_get_user_name ());
+	filedir = g_dir_open (dir, 0, NULL);
+	if (filedir == NULL)
+		return NULL;
 
-	dir = g_getenv ("BACON_SOCKET_DIR");
-	if (dir == NULL)
+	pat = g_pattern_spec_new (pattern);
+	if (pat == NULL)
 	{
-		path = g_build_filename (g_get_home_dir (), filename, NULL);
-	} else {
-		path = g_build_filename (dir, filename, NULL);
+		g_dir_close (filedir);
+		return NULL;
 	}
 
-	g_free (filename);
+	found_filename = NULL;
+
+	while ((filename = g_dir_read_name (filedir)))
+	{
+		if (g_pattern_match_string (pat, filename))
+		{
+			char *tmp = g_build_filename (dir, filename, NULL);
+			if (is_owned_by_user_and_socket (tmp))
+				found_filename = g_strdup (filename);
+			g_free (tmp);
+		}
+
+		if (found_filename != NULL)
+			break;
+	}
+
+	g_pattern_spec_free (pat);
+	g_dir_close (filedir);
+
+	return found_filename;
+}
+
+static char *
+socket_filename (const char *prefix)
+{
+	char *pattern, *newfile, *path, *filename;
+	const char *tmpdir;
+
+	pattern = g_strdup_printf ("%s.%s.*", prefix, g_get_user_name ());
+	tmpdir = g_get_tmp_dir ();
+	filename = find_file_with_pattern (tmpdir, pattern);
+	if (filename == NULL)
+	{
+		newfile = g_strdup_printf ("%s.%s.%u", prefix,
+				g_get_user_name (), g_random_int ());
+		path = g_build_filename (tmpdir, newfile, NULL);
+		g_free (newfile);
+	} else {
+		path = g_build_filename (tmpdir, filename, NULL);
+		g_free (filename);
+	}
+
+	g_free (pattern);
 	return path;
 }
 
@@ -165,7 +254,7 @@ try_server (BaconMessageConnection *conn)
 	uaddr.sun_family = AF_UNIX;
 	strncpy (uaddr.sun_path, conn->path,
 			MIN (strlen(conn->path)+1, UNIX_PATH_MAX));
-	conn->serverfd = conn->fd = socket (PF_UNIX, SOCK_STREAM, 0);
+	conn->fd = socket (PF_UNIX, SOCK_STREAM, 0);
 	if (bind (conn->fd, (struct sockaddr *) &uaddr, sizeof (uaddr)) == -1)
 	{
 		conn->fd = -1;
@@ -175,7 +264,6 @@ try_server (BaconMessageConnection *conn)
 
 	if (!setup_connection (conn))
 		return FALSE;
-	conn->server_conn_id = conn->conn_id;
 	return TRUE;
 }
 
@@ -188,7 +276,6 @@ try_client (BaconMessageConnection *conn)
 	strncpy (uaddr.sun_path, conn->path,
 			MIN(strlen(conn->path)+1, UNIX_PATH_MAX));
 	conn->fd = socket (PF_UNIX, SOCK_STREAM, 0);
-	conn->serverfd = -1;
 	if (connect (conn->fd, (struct sockaddr *) &uaddr,
 				sizeof (uaddr)) == -1)
 	{
@@ -242,13 +329,20 @@ bacon_message_connection_new (const char *prefix)
 void
 bacon_message_connection_free (BaconMessageConnection *conn)
 {
-	g_return_if_fail (conn != NULL);
-	g_return_if_fail (conn->path != NULL);
+	GSList *child_conn;
 
-	if (conn->server_conn_id) {
-		g_source_remove (conn->server_conn_id);
-		conn->server_conn_id = 0;
+	g_return_if_fail (conn != NULL);
+	/* Only servers can accept other connections */
+	g_return_if_fail (conn->is_server != FALSE ||
+			  conn->accepted_connections == NULL);
+
+	child_conn = conn->accepted_connections;
+	while (child_conn != NULL) {
+		bacon_message_connection_free (child_conn->data);
+		child_conn = g_slist_next (child_conn);
 	}
+	g_slist_free (conn->accepted_connections);
+
 	if (conn->conn_id) {
 		g_source_remove (conn->conn_id);
 		conn->conn_id = 0;
@@ -260,8 +354,8 @@ bacon_message_connection_free (BaconMessageConnection *conn)
 
 	if (conn->is_server != FALSE) {
 		unlink (conn->path);
-		close (conn->serverfd);
-	} else if (conn->fd != -1) {
+	}
+	if (conn->fd != -1) {
 		close (conn->fd);
 	}
 
